@@ -71,7 +71,12 @@ typedef struct targelt
 } targelt_t;
 
 
-/** Identifiers of top-level operating mode */
+/** Identifiers of top-level operating mode, and configuration switches
+ *
+ *  The lower-order bits specify one of a sequence of mutuall-exclusive
+ *  operating modes. The higher-order bits consist of a few control flags,
+ *  some of which may be set implicitly according the the operating mode.
+ */
 typedef enum {
     M_UNSET, M_HELP,
     M_PREPARE, M_RELEASE,
@@ -81,18 +86,15 @@ typedef enum {
     M_PASSWORD, M_KEYGEN, M_KEYREU,
     M_SYSBOOT, M_SYSQUIT,
     M_SAFETYNET, M_VERSION,
-    M_SELFTEST
-} cmmode_t;
+    M_SELFTEST,
 
-/**
- *  Configuration switches set (perhaps indirectly)
- *  from command-line options
- */
-enum {
-    F_NEEDS_TGT =   1,    /*!< command requires a filesystem target */
-    F_ALL_TARGETS = 2,    /*!< command will be applied to all known targets */
-    F_VERIFY_PW =   4     /*!< encryption password should be double-checked */
-};
+    M_MODE_MASK = 0x00ff,
+
+    F_NEEDS_TGT =   0x0100, /*!< command requires a filesystem target */
+    F_ALL_TARGETS = 0x0200, /*!< command will be applied to all known targets */
+    F_NOWARN_ALL =  0x0400, /*!< suppress warnings with '--all' targets */
+    F_VERIFY_PW =   0x0800  /*!< encryption password should be double-checked */
+} cmmode_t;
 
 
 static int64_t getblk512count(const char *device, int *blklen);
@@ -336,15 +338,15 @@ static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
         switch (boot_mode) {
             case FLG_BOOT_MOUNT:
                 eflag = (booting ? do_mount(pw_ctxt, boundtgt)
-                                : do_unmount(boundtgt));
+                                 : do_unmount(boundtgt));
                 break;
             case FLG_BOOT_SWAP:
                 eflag = (booting ? do_swapon(pw_ctxt, boundtgt)
-                                : do_swapoff(boundtgt));
+                                 : do_swapoff(boundtgt));
                 break;
             case FLG_BOOT_PREP:
                 eflag = (booting ? do_devsetup(pw_ctxt, boundtgt, NULL)
-                                : do_devshutdown(boundtgt));
+                                 : do_devshutdown(boundtgt));
                 break;
             default:
                 break;
@@ -535,7 +537,7 @@ static int do_mount(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
     if (is_mounted(boundtgt->tgt)) {
         fprintf(stderr, _("Target \"%s\" is already mounted\n"),
                 boundtgt->tgt->ident);
-        eflag = ERR_BADMOUNT;
+        eflag = WRN_MOUNTED;
         goto bail_out;
     }
 
@@ -643,13 +645,20 @@ static int do_unmount(const bound_tgtdefn_t *boundtgt)
  *  \see do_swapoff(), do_devsetup(), fs_swapon().
  */
 static int do_swapon(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
-{   const tgtdefn_t *tgt=NULL;
-    int freedev=0, eflag=ERR_NOERROR;
-    char *mntdev=NULL;
-    tgtstat_t *tstat;
+{   const tgtdefn_t *tgt = NULL;
+    int freedev = 0, eflag = ERR_NOERROR;
+    char *mntdev = NULL;
+    tgtstat_t *tstat = NULL;
 
 #if WITH_CSWAP
 
+    if (is_configured(boundtgt->tgt->ident, NULL)) {
+        fprintf(stderr, _("Target \"%s\" is already configured\n"),
+                        boundtgt->tgt->ident);
+        eflag = WRN_MOUNTED;
+        goto bail_out;
+    }
+
     eflag = do_devsetup(pw_ctxt, boundtgt, &mntdev);
     if (eflag != ERR_NOERROR) goto bail_out;
     tgt = boundtgt->tgt;
@@ -1097,7 +1106,7 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
 #endif
 
     /* Execute special-cases of user-selected task: */
-    switch (mode) {
+    switch ((mode & M_MODE_MASK)) {
         case M_VERSION:
             fprintf(stderr, "%s-%s\n", PACKAGE_NAME, PACKAGE_VERSION);
             break;
@@ -1136,7 +1145,7 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
         syslogmsg = NULL;
         prio = LOG_AUTHPRIV | LOG_NOTICE;
 
-        switch (mode) {
+        switch ((mode & M_MODE_MASK)) {
             case M_PREPARE:
                 syslogmsg = "prepare of \"%s\" by %s %s";
                 eflag = do_devsetup(pw_ctxt, boundtgt, NULL);
@@ -1151,6 +1160,7 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 if ((eflag = check_priv_tgt(boundtgt->tgt)) != ERR_NOERROR) break;
                 syslogmsg = "mount of \"%s\" by %s %s";
                 eflag = do_mount(pw_ctxt, boundtgt);
+                if (eflag == WRN_MOUNTED) syslogmsg = NULL;
                 break;
             case M_UNMOUNT:
                 if ((eflag = check_priv_tgt(boundtgt->tgt)) != ERR_NOERROR) break;
@@ -1161,6 +1171,7 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
             case M_SWAPON:
                 syslogmsg = "swapon \"%s\" by %s %s";
                 eflag = do_swapon(pw_ctxt, boundtgt);
+                if (eflag == WRN_MOUNTED) syslogmsg = NULL;
                 break;
             case M_SWAPOFF:
                 syslogmsg = "swapoff \"%s\" by %s %s";
@@ -1185,6 +1196,13 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
             default:
                 break;
         }
+
+        /* Suppress benign warning messages when '--all' option is selected: */
+        if (eflag != 0 && eflag < ERR_threshold
+                && (mode & F_ALL_TARGETS) && (mode & F_NOWARN_ALL)) {
+            eflag = 0;
+        }
+
 #ifndef TESTING
 #  ifdef HAVE_SYSLOG
         if (syslogmsg != NULL) {
@@ -1255,10 +1273,10 @@ static cmmode_t get_defaultmode(int argc, char *argv[])
  *  only accessible to the super-user.
  */
 cmmode_t parse_options(int argc, char *argv[],
-                       unsigned *flags, const char **mode_params,
+                       const char **mode_params,
                        int *passwd_fd, int *config_fd,
                        km_pw_context_t *pw_ctxt)
-{   cmmode_t mode = M_UNSET;
+{   cmmode_t bare_mode = M_UNSET, mode_flags = 0;
     enum {
         ALL_USERS =     0x01,
         NEEDS_ARG =     0x02,
@@ -1291,13 +1309,13 @@ cmmode_t parse_options(int argc, char *argv[],
         { 'l', "list",              ALL_USERS | SET_MODE,
                                     NULL, M_LIST, 0 },
         { 'm', "mount",             ALL_USERS | SET_MODE | SET_FLAGS,
-                                    NULL, M_MOUNT, F_NEEDS_TGT },
+                                    NULL, M_MOUNT, F_NEEDS_TGT|F_NOWARN_ALL },
         { 'w', "passwd-fd",         ALL_USERS | NEEDS_ARG,
                                     &passwd_fd_str, M_UNSET, 0 },
         { 'p', "prepare",           SET_MODE | SET_FLAGS,
-                                    NULL, M_PREPARE, F_NEEDS_TGT },
+                                    NULL, M_PREPARE, F_NEEDS_TGT|F_NOWARN_ALL },
         { 'r', "release",           SET_MODE | SET_FLAGS,
-                                    NULL, M_RELEASE, F_NEEDS_TGT },
+                                    NULL, M_RELEASE, F_NEEDS_TGT|F_NOWARN_ALL },
         { 'e', "reuse-key",         NEEDS_ARG | SET_MODE | SET_FLAGS,
                                     mode_params, M_KEYREU, F_NEEDS_TGT },
         { 'n', "safetynet",         SET_MODE,
@@ -1305,15 +1323,15 @@ cmmode_t parse_options(int argc, char *argv[],
         { 'S', "status",            ALL_USERS | SET_MODE | SET_FLAGS,
                                     NULL, M_STATUS },
         { 's', "swapon",            SET_MODE | SET_FLAGS,
-                                    NULL, M_SWAPON, F_NEEDS_TGT },
+                                    NULL, M_SWAPON, F_NEEDS_TGT|F_NOWARN_ALL },
         { 'x', "swapoff",           SET_MODE | SET_FLAGS,
-                                    NULL, M_SWAPOFF, F_NEEDS_TGT },
+                                    NULL, M_SWAPOFF, F_NEEDS_TGT|F_NOWARN_ALL },
         { 'B', "system-boot",       SET_MODE,
                                     NULL, M_SYSBOOT, 0 },
         { 'Q', "system-shutdown",   SET_MODE,
                                     NULL, M_SYSQUIT, 0 },
         { 'u', "unmount",           ALL_USERS | SET_MODE | SET_FLAGS,
-                                    NULL, M_UNMOUNT, F_NEEDS_TGT },
+                                    NULL, M_UNMOUNT, F_NEEDS_TGT|F_NOWARN_ALL },
         { 'y', "verify-password",   ALL_USERS | SET_FLAGS,
                                     NULL, M_UNSET, F_VERIFY_PW },  /* FIXME - not implemented? */
         { 'v', "version",           ALL_USERS | SET_MODE,
@@ -1384,8 +1402,8 @@ cmmode_t parse_options(int argc, char *argv[],
         }
 
         if ((selected->flags & SET_MODE)) {
-            if (mode == M_UNSET) {
-                mode = selected->newmode;
+            if (bare_mode == M_UNSET) {
+                bare_mode = selected->newmode;
             } else {
                 fprintf(stderr, _("Multiple operating modes not supported\n"));
                 exit(1);
@@ -1393,7 +1411,7 @@ cmmode_t parse_options(int argc, char *argv[],
         }
 
         if ((selected->flags & SET_FLAGS)) {
-            *flags |= selected->newflags;
+            mode_flags |= selected->newflags;
         }
     }
 
@@ -1402,7 +1420,7 @@ cmmode_t parse_options(int argc, char *argv[],
         exit(EXIT_BADOPT);
     }
 
-    if (mode == M_UNSET) mode = get_defaultmode(argc, argv);
+    if (bare_mode == M_UNSET) bare_mode = get_defaultmode(argc, argv);
     if (config_fd_str != NULL) sscanf(config_fd_str, "%d", config_fd);
     if (passwd_fd_str != NULL) sscanf(passwd_fd_str, "%d", passwd_fd);
 
@@ -1411,13 +1429,12 @@ cmmode_t parse_options(int argc, char *argv[],
 #endif
     free((void*)shortopts);
 
-    return mode;
+    return (bare_mode | mode_flags);
 }
 
 
 int main(int argc, char *argv[])
 {   cmmode_t mode = M_UNSET;
-    unsigned mode_flags = 0;
     const char *mode_params = NULL;
     char *cmtab = NULL;
     int config_fd = -1, passwd_fd = -1, eflag = ERR_NOERROR;
@@ -1448,8 +1465,8 @@ int main(int argc, char *argv[])
     pw_ctxt.verify = 0;
     pw_ctxt.debug_level = 0;
 
-    mode = parse_options(argc, argv, &mode_flags, &mode_params,
-                        &passwd_fd, &config_fd, &pw_ctxt);
+    mode = parse_options(argc, argv, &mode_params,
+                         &passwd_fd, &config_fd, &pw_ctxt);
 
 #ifdef TESTING
     if (mode == M_SELFTEST) {
@@ -1461,7 +1478,7 @@ int main(int argc, char *argv[])
 
     (void)cm_path(&cmtab, CM_SYSCONF_PFX, "cmtab");
 
-    if (mode == M_HELP) {
+    if ((mode & M_MODE_MASK) == M_HELP) {
         fprintf(stderr, "%s", _(USAGE_STRING));
         exit(EXIT_OK);
     }
@@ -1491,12 +1508,13 @@ int main(int argc, char *argv[])
     }
     free((void*)cmtab);
 
-    if ((mode == M_LIST || mode == M_STATUS) && optind >= argc) {
-        mode_flags |= F_ALL_TARGETS;
+    if (((mode & M_MODE_MASK) == M_LIST
+         || (mode & M_MODE_MASK) == M_STATUS) && optind >= argc) {
+        mode |= F_ALL_TARGETS;
     }
 
     /* if '--all' given, assemble list of targets from entire config-file */
-    if ((mode_flags & F_ALL_TARGETS) != 0) {
+    if ((mode & F_ALL_TARGETS) != 0) {
         if (optind < argc) {
             fprintf(stderr, _("Trailing command-line arguments given with '--all' option\n"));
             exit(EXIT_BADOPT);
@@ -1538,7 +1556,7 @@ int main(int argc, char *argv[])
 
 
     /* Execute user-selected task: */
-    if ((mode_flags & F_NEEDS_TGT) && eltlist == NULL) {
+    if ((mode & F_NEEDS_TGT) && eltlist == NULL) {
         fprintf(stderr, _("No targets specified\n"));
         exit(EXIT_BADTGT);
     }
