@@ -72,9 +72,10 @@ enum    /*! Error flags */
 {
     ERR_NOERROR =       0,
 
-    WRN_UNCONFIG,
+    WRN_UNCONFIG,               /*!< Filesystem is already unmounted */
     WRN_NOPASSWD,
     WRN_LOWENTROPY,
+    WRN_MOUNTED,                /*!< Filesystem is already mounted */
 
     ERR_threshold = 0x10,       /*!< Dividing-line between warnings & errors */
 
