@@ -61,7 +61,7 @@ int udev_active_dir(const char *path, time_t starttime, double timeout);
 
 struct dm_task *devmap_prepare(int type, const char *ident)
     /*! Prepare device-mapper task structure */
-{   struct dm_task *dmt=NULL;
+{   struct dm_task *dmt = NULL;
 
     dmt = dm_task_create(type);
     if (dmt != NULL) {
@@ -222,18 +222,19 @@ int devmap_remove(const char *ident)
 
 
 int is_configured(const char *ident, struct dm_info *dminfo)
-    /* check if device-mapper target has been setup & (optionally) get info */
-{   struct dm_task *dmt=NULL;
+    /*! Check if device-mapper target has been setup & (optionally) get info */
+{   struct dm_task *dmt = NULL;
     struct dm_info *dmi, dmi_local;
-    int config=1;
+    int config = 1;
 
     dmi = (dminfo != NULL ? dminfo : &dmi_local);
 
-    /* create device-mapper target: */
+    /* Create device-mapper target: */
     if (ident == NULL
       || (dmt = devmap_prepare(DM_DEVICE_INFO, ident)) == NULL
       || !dm_task_run(dmt)
-      || !dm_task_get_info(dmt, dmi)) {
+      || !dm_task_get_info(dmt, dmi)
+      || !dmi->exists) {
         config = 0;
     }
     if (dmt != NULL) dm_task_destroy(dmt);
